# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements a class responsible for the file browser side panel.
"""

import gtk
import gnomevfs
import shutil
import string
import gettext
import re

from TextFlow.com.Files import FileList
from TextFlow.core.PreferencesManager import PreferencesManager
from TextFlow.widgets.FilterWindow import FilterWindow
from TextFlow.core import constants

_ = gettext.gettext

#The name of this side panel
name = _("File Browser")

class FileBrowser(gtk.VBox):
    """
    This class is responsible for the file browser side panel.
    """
    def __init__(self, document_manager, sidepanel):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        
        @param sidepanel: Reference to the sidepanels container.
        @type sidepanel: A VBox object.
        """
        super(FileBrowser, self).__init__()
        self.document_manager = document_manager
        self.sidepanel = sidepanel
        self.preferences_manager = PreferencesManager()
        self.expanded_paths = []
        
        self.except_mimes = ("application/x-javascript", 'application/x-shellscript',  'application/x-python', 'application/x-php-source', 'application/x-perl', 'application/x-awk', 'application/x-m4', 'application/x-ini-file', 'application/x-tcl', 'application/x-gnome-app-info', 'application/x-desktop', 'application/docbook+xml', "application/xml", "application/x-php",
                             "application/x-asp", "application/x-ruby")
        
        self.treestore = gtk.TreeStore(gtk.gdk.Pixbuf, str)
        
        self.treeview = gtk.TreeView(self.treestore)
        self.treeview.set_headers_visible(False)
        
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC,
                                        gtk.POLICY_ALWAYS)
        self.scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        self.scrolled_window.add(self.treeview)
        
        self.__set_bar()
#        self.__set_filter()
        
        self.pack_start(self.scrolled_window)
#        self.pack_start(self.expander, False, False)
        self.pack_start(self.bar, False, False)
        
        start_dir = self.preferences_manager.get_value("filebrowser_dir")
        
        self.flist = FileList(start_dir)
        
        self.tvcolumn = gtk.TreeViewColumn()
        self.cell = gtk.CellRendererPixbuf()
        self.cell2 = gtk.CellRendererText()

        self.tvcolumn.pack_start(self.cell, False)
        self.tvcolumn.pack_start(self.cell2, True)
        self.tvcolumn.add_attribute(self.cell, 'pixbuf', 0)
        self.tvcolumn.add_attribute(self.cell2, 'text', 1)
        
        self.treeview.append_column(self.tvcolumn)
        self.treeview.set_search_column(1)
        
        self.treeview.connect('row-expanded', self.expand_folder)
        self.treeview.connect('row-collapsed', self.collapsed_folder)
        self.treeview.connect('row-activated', self.double_click)
        self.treeview.connect('key-press-event', self.key_press)
        self.cell2.connect('edited', self.edit_file_name)
        
        self.__put_files()
        
#    def __set_filter(self):
#        """
#        Create the filter expand and entry.
#        """
#        self.expander = gtk.Expander(_("Filter"))
#        self.filter_entry = gtk.Entry()
#        self.expander.add(self.filter_entry)
#        self.filter_entry.connect("activate", self.filter_activate)
        
    def __set_bar(self):
        """
        Create the bottom bar.
        """
        self.bar = gtk.HBox()
        self.tooltip = gtk.Tooltips()
        
        image_new = gtk.Image()
        image_new.set_from_stock(gtk.STOCK_NEW, gtk.ICON_SIZE_MENU)
        
        image_open = gtk.Image()
        image_open.set_from_stock(gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_MENU)
        
        image_up = gtk.Image()
        image_up.set_from_stock(gtk.STOCK_GO_UP, gtk.ICON_SIZE_MENU)
        
        image_home = gtk.Image()
        image_home.set_from_stock(gtk.STOCK_HOME, gtk.ICON_SIZE_MENU)
        
        image_refresh = gtk.Image()
        image_refresh.set_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_MENU)
        
        image_filter = gtk.Image()
        image_filter.set_from_stock(gtk.STOCK_FIND, gtk.ICON_SIZE_MENU)
        
        self.button_new = gtk.Button()
        self.button_new.set_image(image_new)
        self.button_new.set_property('relief', gtk.RELIEF_NONE)
        self.tooltip.set_tip(self.button_new, _("New File"))
        
        self.button_open = gtk.Button()
        self.button_open.set_image(image_open)
        self.button_open.set_property('relief', gtk.RELIEF_NONE)
        self.tooltip.set_tip(self.button_open, _("New Folder"))
        
        self.button_up = gtk.Button()
        self.button_up.set_image(image_up)
        self.button_up.set_property('relief', gtk.RELIEF_NONE)
        self.tooltip.set_tip(self.button_up, _("Up"))
        
        self.button_home = gtk.Button()
        self.button_home.set_image(image_home)
        self.button_home.set_property('relief', gtk.RELIEF_NONE)
        self.tooltip.set_tip(self.button_home, _("Home"))
        
        self.button_refresh = gtk.Button()
        self.button_refresh.set_image(image_refresh)
        self.button_refresh.set_property('relief', gtk.RELIEF_NONE)
        self.tooltip.set_tip(self.button_refresh, _("Refresh"))
        
        self.button_filter = gtk.Button()
        self.button_filter.set_image(image_filter)
        self.button_filter.set_property('relief', gtk.RELIEF_NONE)
        self.tooltip.set_tip(self.button_filter, _("Filter"))
        
        self.bar.pack_start(self.button_new, False, False)
        self.bar.pack_start(self.button_open, False, False)
        self.bar.pack_start(self.button_up, False, False)
        self.bar.pack_start(self.button_home, False, False)
        self.bar.pack_start(self.button_refresh, False, False)
        self.bar.pack_start(self.button_filter, False, False)
        
        self.button_new.connect('clicked', self.new_file)
        self.button_open.connect('clicked', self.new_folder)
        self.button_up.connect('clicked', self.up_folder)
        self.button_home.connect('clicked', self.go_home)
        self.button_refresh.connect('clicked', self.refresh_tree)
        self.button_filter.connect('clicked', self.filter)
        
        self.filter_text = ""
        
    def __put_files(self, tree_iter=None, show_hidden=False):
        """
        Put files of the current directory starting in tree_iter.
        
        @param tree_iter: The tree iter where the will be put.
        @type tree_iter: A TreeIter object.
        
        @param show_hidden: True if this function put hidden files too.
        @type show_hidden: A boolean.
        
        @return: True if the current directory has some files,
        False if the current directory is empty.
        @rtype: A boolean.
        """
        filter = self.__filter_value()
        
        directory = self.flist.get_current_dir()
        theme = gtk.icon_theme_get_default()
        self.treeview.set_model(self.treestore)
        
        if tree_iter == None:
            icon = \
            self.treeview.render_icon(gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_MENU)
            direc = self.__get_current_directory(directory)
            tree_iter = self.treestore.append(tree_iter, [icon, direc])
            
        files = self.flist.get_files(show_hidden)
        
        if len(files) == 0:
            self.treeview.columns_autosize()
            
            try:
                self.treestore.get_iter((0, 0))
            except ValueError:
                tree_iter = self.treestore.get_iter((0,))
                self.treestore.append(tree_iter, [None, _('(Empty)')])
                self.treeview.expand_row((0,), False)
            
            return False
        else:
            info = theme.lookup_icon("text-x-generic", gtk.ICON_SIZE_MENU, 0)
            icon_f = info.load_icon()
            
            for i in files:
                if i.type == gnomevfs.FILE_TYPE_DIRECTORY:
                    icon = self.treeview.render_icon(gtk.STOCK_DIRECTORY,
                                                     gtk.ICON_SIZE_MENU)
                    direc = self.__get_current_directory(directory)
                    i_iter = self.treestore.append(tree_iter, [icon, i.name])
                    self.treestore.append(i_iter, [None, _('(Empty)')])
                elif i.type == gnomevfs.FILE_TYPE_REGULAR:
                    url = \
                    gnomevfs.get_uri_from_local_path(directory + "/" + i.name)
                    mime = gnomevfs.get_mime_type(url)
                    t = mime.split("/")
                    if mime in self.except_mimes or t[0] == "text":
                        if filter == "":
                            i_iter = self.treestore.append(tree_iter, [icon_f, i.name])
                        else:
                            if self.__parse_filter(i.name, filter) != None:
                                i_iter = self.treestore.append(tree_iter, [icon_f, i.name])
                            
            self.treeview.columns_autosize()

            self.treeview.expand_row((0,), False)

            return True
    
    def refresh(self):
        self.treestore.clear()
        self.__put_files(None)
        for i in self.expanded_paths:
            self.treeview.expand_row(i, False)

    def expand_folder(self, widget, tree_iter, path):
        """
        Show the files of a expanded folder.
        
        @param widget: Reference to the TreeView
        @type widget: A TreeView object.
        
        @param tree_iter: A TreeIter that points to the expanded folder.
        @type tree_iter: A TreeIter object.
        
        @param path: The path for the expanded folder.
        @type path: A tuple.
        """
        child = self.treestore.iter_children(tree_iter)
        
        if not (path in self.expanded_paths):
            self.expanded_paths.append(path)
        if (self.treestore.get_value(child, 1) == _('(Empty)')):
            #Setting directory
            current_dir = self.flist.get_current_dir()
            subfolder_dir = current_dir
            
            if subfolder_dir == "/":
                subfolder_dir = ""
            
            #Get directory from path
            path = self.treestore.get_path(child)

            for i in range(2, len(path)):
                subfolder = self.treestore.get_iter(path[:i])
                subfolder_dir += "/" + self.treestore.get_value(subfolder, 1)
            
            self.flist.set_current_dir(subfolder_dir)
            
            #Add files to the expanded folder
            n = self.treestore.iter_n_children(tree_iter)
            if n == 1:
                if self.__put_files(tree_iter):
                    self.treestore.remove(child)
            self.flist.set_current_dir(current_dir)

    def collapsed_folder(self, widget, tree_iter, path):
        self.expanded_paths.remove(path)
    
    def double_click(self, widget, path, view_column):
        """
        Show the files of a expanded folder.
        
        @param widget: Reference to the TreeView
        @type widget: A TreeView object.
        
        @param path: The path for the expanded folder.
        @type path: A tuple.
        
        @param view_column: The column in the activated row.
        @type view_column: A TreeViewColumn object.
        """
#        tree_iter = self.treestore.get_iter(path)
#        folder_name = self.treestore.get_value(tree_iter,1)
        
        #Setting directory
        subfolder_dir = self.flist.get_current_dir()
        if subfolder_dir == "/":
            subfolder_dir = ""
        
        #Get directory from path
        for i in range(2, len(path)+1):
            subfolder = self.treestore.get_iter(path[:i])
            subfolder_dir += "/" + self.treestore.get_value(subfolder, 1)
        
        if subfolder_dir == "":
            subfolder_dir = "/"
            
#        subfolder_name = self.__get_current_directory(subfolder_dir)
        
        file_clicked = gnomevfs.get_file_info(subfolder_dir)
        
        if file_clicked.type == gnomevfs.FILE_TYPE_DIRECTORY:
            self.flist.set_current_dir(subfolder_dir)
            self.treestore.clear()
#            not_empty = self.__put_files(None)
            self.__put_files(None)
            
        elif file_clicked.type == gnomevfs.FILE_TYPE_REGULAR:
            self.document_manager.open_file_tab(subfolder_dir)
    
    def up_folder(self, widget):
        """
        Show the files of the upper folder.
        
        @param widget: Reference to the button
        @type widget: A Button object.
        """
        current_dir_list = self.flist.get_current_dir().split("/")
        
        current_dir = string.join(current_dir_list[:-1], "/")
        
        if current_dir == "":
            self.flist.set_current_dir("/")
        else:
            self.flist.set_current_dir(current_dir)
        
        self.treestore.clear()
        self.__put_files(None)
        
    def go_home(self, widget):
        self.flist.set_current_dir(constants.HOME)
        self.treestore.clear()
        self.__put_files(None)
        
    def new_file(self, widget):
        """
        Create a new file in the current directory
        
        @param widget: Reference to the button
        @type widget: A Button object.
        """
        
        #Create file
        i = 1
        new_file_name = _("New File")
        new_file_uri = self.flist.get_current_dir() + '/' + new_file_name

        while gnomevfs.exists(new_file_uri):
            new_file_uri = \
            self.flist.get_current_dir() + '/' + new_file_name + " " + str(i)
            i += 1

        files = self.flist.get_files(False)
        
        new_file = gnomevfs.create(new_file_uri, gnomevfs.OPEN_WRITE)
        new_file.close()
        
        tree_iter = self.treestore.get_iter((0,))
        icon = self.treeview.render_icon(gtk.STOCK_FILE,
                                         gtk.ICON_SIZE_MENU)
        name = gnomevfs.URI(new_file_uri).short_name
        new_iter = self.treestore.append(tree_iter, [icon, name])
        
        #Verifying if the directory is empty to delete "(Empty)" string.
        if len(files) == 0:
            empty_iter = self.treestore.get_iter((0, 0))
            self.treestore.remove(empty_iter)
        
        path = self.treestore.get_path(new_iter)
        self.cell2.set_property('editable', True)
        self.treeview.set_cursor_on_cell(path, self.tvcolumn, self.cell2,
                                         start_editing=True)
        self.cell2.set_property('editable', False)
    
    def edit_file_name(self, widget, path, new_text):
        current_dir = self.flist.get_current_dir()
        root_iter = self.treestore.get_iter_first()
        file_iter = self.treestore.get_iter_from_string(path)
        tuple_path = self.treestore.get_path(file_iter)
        old_file_name = self.treestore.get_value(file_iter, 1)
        file_dir = current_dir
        
        for i in tuple_path[1:-1]:
            root_iter = self.treestore.iter_nth_child(root_iter, i)
            file_dir += '/' + self.treestore.get_value(root_iter, 1)

        if new_text != old_file_name:

            old_file_uri = file_dir + '/' + old_file_name
            new_file_uri = file_dir + '/' + new_text

            if gnomevfs.exists(new_file_uri):
                error = RenameErrorDialog()
                error_return = error.run()
                
                if error_return == gtk.RESPONSE_DELETE_EVENT:
                    error.destroy()
                    return
                elif error_return == 1:
                    error.destroy()
                    return
            
            old_file_info = gnomevfs.get_file_info(old_file_uri)
            old_file_info.name = new_text
            gnomevfs.set_file_info(old_file_uri, old_file_info,
                                   gnomevfs.SET_FILE_INFO_NAME)
            
            self.treestore.set_value(file_iter, 1, new_text)
        self.refresh()

    def new_folder(self, widget):
        """
        Create a new folder in the current directory
        
        @param widget: Reference to the button
        @type widget: A Button object.
        """
        
        #Create file
        i = 1
        new_file_name = _("New Folder")
        new_file_uri = self.flist.get_current_dir() + '/' + new_file_name

        while gnomevfs.exists(new_file_uri):
            new_file_uri = \
            self.flist.get_current_dir() + '/' + new_file_name + " " + str(i)
            i += 1

        gnomevfs.make_directory(new_file_uri, 0755)
        
        tree_iter = self.treestore.get_iter((0,))
        icon = self.treeview.render_icon(gtk.STOCK_DIRECTORY,
                                         gtk.ICON_SIZE_MENU)
        name = gnomevfs.URI(new_file_uri).short_name
        new_iter = self.treestore.append(tree_iter, [icon, name])
        path = self.treestore.get_path(new_iter)
        self.cell2.set_property('editable', True)
        self.treeview.set_cursor_on_cell(path, self.tvcolumn, self.cell2,
                                         start_editing=True)
        self.cell2.set_property('editable', False)

    def refresh_tree(self, widget):
        self.refresh() 

    def filter(self, widget):
        alloc = widget.get_allocation()
        
        wp_x, wp_y = self.window.get_origin()
        
        pw = FilterWindow()
        pw.entry.set_text(self.filter_text)
        pw.entry.connect('activate', self.filter_activate, pw)
        pw.run((alloc.x + wp_x - 80, alloc.y + wp_y - 60))

    def delete_file(self, path, iter):
        home_dir = constants.HOME
        current_dir = self.flist.get_current_dir()
        root_iter = self.treestore.get_iter_first()
        file_name = self.treestore.get_value(iter, 1)
        file_dir = current_dir
        
        for i in path[1:]:
            root_iter = self.treestore.iter_nth_child(root_iter, i)
            file_dir += '/' + self.treestore.get_value(root_iter, 1)

        
        file_dir_url = gnomevfs.get_uri_from_local_path(file_dir)
        file_clicked = gnomevfs.get_file_info(file_dir_url)
        
        if file_clicked.type == gnomevfs.FILE_TYPE_DIRECTORY:
            shutil.move(file_dir, home_dir + "/.Trash/" + file_name)
            self.treestore.remove(iter)

        elif file_clicked.type == gnomevfs.FILE_TYPE_REGULAR:
            shutil.move(file_dir, home_dir + "/.Trash/" + file_name)
            self.treestore.remove(iter)
            
        self.refresh()

    def key_press(self, widget, event):
#         print event.keyval
        sel_model, sel_iter = widget.get_selection().get_selected()
        if event.keyval == 65471:
            #F2 to rename
            if sel_iter != None:
                path = self.treestore.get_path(sel_iter)
                self.cell2.set_property('editable', True)
                self.treeview.set_cursor_on_cell(path, self.tvcolumn, 
                                                 self.cell2,
                                                 start_editing=True)
                self.cell2.set_property('editable', False)
        elif event.keyval == 65474:
            #F5 to refresh
            self.refresh()
        elif event.keyval == 65535:
            #Del to delete
            if sel_iter != None:
                path = self.treestore.get_path(sel_iter)
                self.delete_file(path, sel_iter)

    def filter_activate(self, widget, window):
        """
        Apply filter.
        
        @param widget: A reference for a Entry.
        @type widget: A Entry object.
        """
        self.filter_text = widget.get_text()
        window.destroy()
        self.refresh()

    def __parse_filter(self, value, f):
        """
        Verify if f filters value.
        
        @param value: A text that will be filtered
        @type value: A string.
        
        @param f: A text filter.
        @type f: A string.
        
        @return: If value pass through filter returns value, else return "".
        @rtype: A string.
        """
        result = None
		
        f = f.replace("*", ".+")
        f = f.replace("\.+", "\*")
		
        if value != "":
            filter = re.compile("^" + f, re.IGNORECASE)
            matches = filter.findall(value)
            if len(matches) > 0:
                result = matches
            		
        return result
		
    def __filter_value(self):
        """
        This method returns the filter if it is valid.
        """
        return self.filter_text
#        
#        if text != "":
#            return text
#        else:
#            return ""
        
    def __get_current_directory(self, dir_path):
        """
        Get the name of the directory of the path defined by dir_path.
        
        @param dir_path: A path for a directory.
        @type dir_path: A string.
        """
        dir_list = dir_path.split('/')
        directory = dir_list[len(dir_list) - 1]
        if directory == "":
            return "/"
        else:
            return directory
            
    def unload(self):
        c_dir = self.flist.get_current_dir()
        self.preferences_manager.set_value("filebrowser_dir", c_dir)
    
class RenameErrorDialog(gtk.MessageDialog):
    
    def __init__(self, parent=None):
        super(RenameErrorDialog, self).__init__(parent, 0,
                                                gtk.MESSAGE_QUESTION,
                                                gtk.BUTTONS_NONE, None)

        self.format_secondary_text(_("File already exists."))
        self.set_markup(_("<b>A file with the same name already exists. Please use a different name.</b>"))
        self.add_button(gtk.STOCK_OK, 1)
        


